﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TelemetryIO.Models
{
    public sealed class Telemetry
    {
        private static volatile Telemetry instance;
        private static readonly object _lock = new object();
        private Dictionary<string, VarAddress> dictMonitor = new Dictionary<string, VarAddress>();
        public string name = "";
        public float[] raw_imu = new float[6];
        public float[] quaternion = new float[4];
        public float[] motor_act = new float[4];
        public float[] motor_sp = new float[4];
        public float[] pid0 = new float[9];
        public float[] pid1 = new float[9];
        public float[] pid2 = new float[9];
        public float[] pid3 = new float[9];

        private Telemetry() {
            dictMonitor.Add("rawAx", new VarAddress(0, 0));
            dictMonitor.Add("rawAy", new VarAddress(0, 1));
            dictMonitor.Add("rawAz", new VarAddress(0, 2));
            dictMonitor.Add("rawGx", new VarAddress(0, 3));
            dictMonitor.Add("rawGy", new VarAddress(0, 4));
            dictMonitor.Add("rawGz", new VarAddress(0, 5));

            dictMonitor.Add("Qw", new VarAddress(1, 0));
            dictMonitor.Add("Qx", new VarAddress(1, 1));
            dictMonitor.Add("Qy", new VarAddress(1, 2));
            dictMonitor.Add("Qz", new VarAddress(1, 3));

            dictMonitor.Add("motor0_Act", new VarAddress(100, 0));
            dictMonitor.Add("motor1_Act", new VarAddress(100, 1));
            dictMonitor.Add("motor2_Act", new VarAddress(100, 2));
            dictMonitor.Add("motor3_Act", new VarAddress(100, 3));
            dictMonitor.Add("motor4_Act", new VarAddress(100, 4));
            dictMonitor.Add("motor5_Act", new VarAddress(100, 5));
            dictMonitor.Add("motor6_Act", new VarAddress(100, 6));
            dictMonitor.Add("motor7_Act", new VarAddress(100, 7));
        }

        public VarAddress getVarAdress(string name)
        {
            VarAddress ret;
            dictMonitor.TryGetValue(name, out ret);
            return ret;
        }

        public string[] getKeys()
        {
            return dictMonitor.Keys.ToArray();
        }
        
        public static Telemetry Instance
        {
            get
            {
                if(instance == null)
                {
                    lock(_lock)
                    {
                        if (instance == null)
                        {
                            instance = new Telemetry();
                        }
                    }
                }
                return instance;
            }
        }

        public void setSlot(int slot, byte[] data, int offset, int len)
        {
            if (slot != 9999) Debug.WriteLine(slot);
            switch (slot)
            {
                case 0://RAW IMU
                    set_float(raw_imu, data, offset, len);
                    break;

                case 1://quaternion
                    set_float(quaternion, data, offset, len);
                    break;

                case 100://motors act
                    set_float(motor_act, data, offset, len);
                    break;

                case 1000:
                    set_float(pid0, data, offset, len);
                    break;

                case 1001:
                    set_float(pid1, data, offset, len);
                    break;

                case 1002:
                    set_float(pid2, data, offset, len);
                    break;

                case 1003:
                    set_float(pid3, data, offset, len);
                    break;

                case 9999:
                    set_name(data);
                    break;
            }
        }


        private void set_float(float[] dest, byte[] data, int offset, int len)
        {
            for (int i = 0; i < (int)(len / 4); i++)
            {
                byte[] bs = new byte[4];
                Array.Copy(data, i * 4, bs, 0, 4);
                float f = BitConverter.ToSingle(bs, 0);
                dest[offset + i] = f;
            }
        }

        private void set_name(byte[] data)
        {
            name = Encoding.ASCII.GetString(data, 0, 16);
        }

    }

    public class VarAddress
    {
        public int slot = 0;
        public int offset = 0;
        public int length = 0;

        public VarAddress()
        {
            slot = offset = 0;
            length = 4;
        }

        public VarAddress(int slot, int offset, int length = 4)
        {
            this.slot = slot;
            this.offset = offset;
            this.length = length;
        }
    }
}
