﻿using ScottPlot;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class MonitorVars : UserControl
    {
        public SerialHandler serial { get; set; }
        public float[][] monitoringBuffer = new float[32][];
        private List<double[]> monitorList = new List<double[]>();
        private List<string> monitorNames = new List<string>();
        private List<ScottPlot.Plottables.Signal> signals = new List<ScottPlot.Plottables.Signal>();

        private List<int[]>[] plotContent = new List<int[]>[32];
        
        private Timer updateTimer = new Timer();
        private float[] monitoring = new float[32];
        private int test_index = 0;
        private bool isSubscribed = false;

        private string processingName = "";
        private int processingIndex = -1;
        public MonitorVars()
        {
            InitializeComponent();
            initMonitoringBuffer();
            
            updateTimer.Interval = 100;

            MainFormPlot.Plot.Axes.AutoScale();
            
            monitorItems_cb.Items.AddRange(Telemetry.Instance.getKeys());
            monitorItems_cb.SelectedIndex = 0;
            updateTimer.Tick += UpdateTimer_Tick;
            updateTimer.Start();
            //MainFormPlot.
        }

        public void subscribeOnMonitoringItemsReceived(SerialHandler s)
        {
            if (isSubscribed) return;
            isSubscribed = true;
            serial = s;
            s.MonitoringItemsReceived += Serial_MonitoringItemsReceived;
            s.AnswerReceived += S_AnswerReceived;
        }

        private void S_AnswerReceived(object sender, SerialEventArgs e)
        {
            if(e.Answer == "ADD")
            {
                monitorNames.Add(processingName);
                processingIndex++;
                if (monitorItems_lb.InvokeRequired)
                {
                    monitorItems_lb.Invoke(new Action(() => monitorItems_lb.Items.Add(processingName)));
                }
                else
                {
                    monitorItems_lb.Items.Add(processingName);
                }
                monitorList.Add(new double[1024]);
                MainFormPlot.Plot.Add.Signal(monitorList[monitorList.Count - 1]);
                
            }
            else if(e.Answer == "REMOVE")
            {
                monitorNames.RemoveAt(processingIndex);
                monitorList.RemoveAt(processingIndex);
                if (monitorItems_lb.InvokeRequired)
                {
                    monitorItems_lb.Invoke(new Action(() => monitorItems_lb.Items.RemoveAt(processingIndex)));
                }
                else
                {
                    monitorItems_lb.Items.RemoveAt(processingIndex);
                }
                
                if (processingIndex >= 0) processingIndex--;
            }else if(e.Answer == "REMOVEALL")
            {
                monitorNames.Clear();
                monitorList.Clear();
            }
        }

        private void Serial_MonitoringItemsReceived(object sender, MonitoringEventArgs e)
        {
            for(int i = 0; i < monitorList.Count; i++)
            {
                ShiftLeft(monitorList[i]);
                monitorList[i][monitorList[i].Length - 1] = e.Data[i];
            }

            monitoring = new float[monitorList.Count];
            for (int i = 0; i < monitorList.Count; i++)
            {
                monitoring[i] = (float)monitorList[i][monitorList[i].Length - 1];
            }
        }

        private void UpdateTimer_Tick(object sender, EventArgs e)
        {
            if (mon_test_label.InvokeRequired)
            {
                mon_test_label.Invoke(new Action(() => mon_test_label.Text = string.Join(", ", monitoring)));
            }
            else
            {
                mon_test_label.Text = string.Join(", ", monitoring);
            }

            if (monNames_label.InvokeRequired)
            {
                monNames_label.Invoke(new Action(() => monNames_label.Text = string.Join(", ", monitorNames)));
            }
            else monNames_label.Text = string.Join(", ", monitorNames);

            if (monitorList.Count > 0)
            {
                MainFormPlot.Plot.Clear();
                for(int i = 0; i < monitorList.Count; i++)
                {
                    var sig = MainFormPlot.Plot.Add.Signal(monitorList[i]);
                    sig.LegendText = monitorNames[i];
                }
                MainFormPlot.Plot.Legend.IsVisible = true;
                MainFormPlot.Plot.Legend.Orientation = ScottPlot.Orientation.Horizontal;
                MainFormPlot.Plot.Axes.AutoScale();
                MainFormPlot.Refresh();
            }
        }

        void initMonitoringBuffer()
        {
            for (int i = 0; i < monitoringBuffer.Length; i++)
            {
                monitoringBuffer[i] = new float[8192];
            }
        }

        private void add_btn_Click(object sender, EventArgs e)
        {
            processingName = (string)monitorItems_cb.SelectedItem;
            serial.AddMonitoringItem(processingName);
            /*serial.AddMonitoringItem(0, test_index);
            processingName = "item." + test_index;
            if (test_index < 5) test_index++;
            else test_index = 0;*/
        }

        private void del_btn_Click(object sender, EventArgs e)
        {
            processingIndex = monitorItems_lb.SelectedIndex;
            serial.RemoveMonitoringItem(processingIndex);
        }

        private void ShiftLeft(double[] arr)
        {
            Buffer.BlockCopy(arr, sizeof(double), arr, 0, (arr.Length - 1)*sizeof(double));
        }
    }
}
